import json
from datetime import datetime, timezone

from clicksend_client import ApiClient
from clicksend_client import Configuration as BaseConfiguration
from clicksend_client import SMSApi, SmsMessage, SmsMessageCollection
from clicksend_client.rest import ApiException
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from ..exceptions import OutboundMessageError
from ..message import Message, MessageType
from .abc import Receiver as BaseReceiver
from .abc import Sender as BaseSender

__all__ = ("Receiver", "Sender")

MAX_BATCH_SIZE = 1000  # see https://developers.clicksend.com/docs/rest/v3/#how-many-messages-can-i-send


class Configuration(BaseConfiguration):
    def __init__(self):
        super().__init__()

        self.username = settings.SMS_SETTINGS["clicksend_username"]
        self.password = settings.SMS_SETTINGS["clicksend_password"]

        try:
            settings.SMS_SETTINGS["clicksend_sender_id"]
        except KeyError:
            raise ImproperlyConfigured(
                "'clicksend_sender_id' must be set in SMS_SETTINGS for ClickSend backend"
            )


client = ApiClient(Configuration())


class Receiver(BaseReceiver):
    fetch = None

    def handle(self, timestamp="", body="", message_id="", **kwargs):
        try:
            timestamp = int(timestamp)
        except ValueError:
            raise ValueError("invalid timestamp")

        if not message_id:
            raise ValueError("empty message id")

        yield Message(
            message_id,
            sender=kwargs["from"],
            text=body,
            type=MessageType.INBOUND,
            created_at=datetime.fromtimestamp(timestamp, timezone.utc),
        )


class Sender(BaseSender):
    def send(self, messages):
        messages = messages[:MAX_BATCH_SIZE]

        try:
            response = (
                SMSApi(client)
                .sms_send_post(
                    SmsMessageCollection(
                        messages=[
                            SmsMessage(
                                **{
                                    "from": settings.SMS_SETTINGS[
                                        "clicksend_sender_id"
                                    ],
                                    "body": message.text,
                                    "to": message.recipient,
                                    "source": "shiftregister",
                                    "custom_string": message.key,
                                }
                            )
                            for message in messages
                        ]
                    )
                )
                .data
            )
        except ApiException as e:
            if e.body:
                response = e.body
            else:
                raise OutboundMessageError(f"{e.status} {e.reason}")

        response = json.loads(response)

        for message in response.get("messages", []):
            if message["status"] == "SUCCESS":
                yield Message(
                    message["custom_string"],
                    recipient=message["to"],
                    sender=message["from"],
                    text=message["body"],
                )
